%% Define the basic data structure of the message.
%% All the rules have 'msg' in them to specify that these rules depend on the msg structure
% a(to, from, about, type, purpose, in Reply to, consented by, belief)
% Belief is also a list with the following list
% b(about, what, believed by)
% One for the person and one for the role.
%consented by is a tuple : (person,consent_type)
 

msg_to(a(X,_,_,_,_,_,_,_), X).

is_msg_to_role(A, Y):-
  msg_to(A, X),
  %in_role(X, Y).
  in_role_closures(X, Y).

msg_to_concerned(a(X,_,X,_,_,_,_,_), X).

is_msg_to_concerned_role(A, Y):-
  msg_to_concerned(A, X),
  %in_role(X, Y).
  in_role_closures(X, Y).

msg_from(a(_,X,_,_,_,_,_,_), X).

is_msg_from_role(A, Y) :-
  msg_from(A, X),
  %in_role(X, Y).
  in_role_closures(X, Y).

msg_from_concerned(a(_,X,X,_,_,_,_,_), X).

is_msg_from_concerned_role(A, Y):-
  msg_from_concerned(A, X),
  %in_role(X, Y).
  in_role_closures(X, Y).

msg_about(a(_,_,X,_,_,_,_,_), X).

is_msg_about_role(A, Y):-
  msg_about(A, X),
  %in_role(X, Y).
  in_role_closures(X, Y).

is_msg_type(a(_,_,_,X,_,_,_,_), Y):-
in_type_closures(X,Y).

has_msg_purpose(a(_,_,_,_,X,_,_,_), X).

is_msg_replyto(a(_,_,_,_,_,X,_,_),X).

is_msg_consented(a(_,_,_,_,_,_,(X,consented),_),X).

is_msg_consent_opp_given(a(_,_,_,_,_,_,(X,opp_given),_),X).


msg_belief(a(_,_,_,_,_,_,_,b(X,Y,Z)), X,Y,Z).

has_msg_belief(A, ABT, WHAT, BY):-
  msg_belief(A, ABT, WHAT, BY).

is_msg_to_within(a(X,Y,_,_,_,_,_,_)):-
  employee_of(X, Z),
  employee_of(Y, Z).

is_msg_about_to_inRelation(a(Y,_,X,_,_,_,_,_)):-
%in relation is not commutative and should be careful which way
%in 164.506 Y has or has had relationship with X
  in_relation(X, Y).


%is_msg_minimum_necessary(A, B) :-
%msg_from(A),
%consent_from_covered_entity(B).
  
:- ['basic-message-structure.pl'].
%% this file provides wrappers to call the basic routines.
%% all rules start with 'is'

%%%%%%%%%%% FROM
is_from_coveredEntity(A) :-
  is_msg_from_role(A, covered_entity).

is_from_employeeOf(A, Y) :-
  msg_from(A, X),
  employee_of(X, Y).

is_from_businessAssociateOf(A) :-
  msg_from(A, X),
  msg_to(A, M),
  business_associate(X, M).

is_from_businessAssociateOf(A, Y) :-
  msg_from(A, X),
  business_associate(X, Y).

is_from_healthCareProvider(A) :-
  is_msg_from_role(A, healthCare_provider).
  %is_from_employeeOf(A, Y),
  %in_role(Y, healthCare_provider).

is_from_healthPlan(A) :-
  is_msg_from_role(A, health_plan).
  %is_from_employeeOf(A, Y),
  %in_role(Y, health_plan).

is_from_healthInsuranceIssuer(A) :-
  is_msg_from_role(A, health_insurance_issuer).
  %is_from_employeeOf(A, Y),
  %in_role(Y, health_insurance_issuer).


is_from_groupHealthPlan(A) :-
  is_msg_from_role(A, group_health_plan).
  %is_from_employeeOf(A, Y),
  %in_role(Y, group_health_plan).

is_from_governmentAgencyHealthPlan(A) :-
  is_msg_from_role(A, government_agency_health_plan).
  %is_from_employeeOf(A, Y),
  %in_role(Y, government_agency_health_plan).

is_from_concernedIndividual(A) :-
  is_msg_from_concerned_role(A, individual).

%%%%%%%%%%% To
is_to_coveredEntity(A) :-
  is_msg_to_role(A, covered_entity).

is_to_employeeOf(A, Y) :-
  msg_to(A, X),
  employee_of(X, Y).

is_to_businessAssociateOf(A) :-
  msg_to(A, X),
  msg_from(A, M),
  business_associate(X, M).

is_to_businessAssociateOf(A, Y) :-
  msg_to(A, X),
  business_associate(X, Y).
  
is_to_healthOversightAgency(A) :-
  is_msg_to_role(A, health_oversight_agency).
  %is_to_employeeOf(A, Y),
  %in_role(Y, health_oversight_agency).

is_to_publicHealthAuthority(A) :-
  is_msg_to_role(A, public_health_authority).
  %is_to_employeeOf(A, Y),
  %in_role(Y, public_health_authority).

is_to_healthCareAccreditationOrganization(A) :-
  is_msg_to_role(A, healthCare_accreditation_organization).
  %is_to_employeeOf(A, Y),
  %in_role(Y, healthCare_accreditation_organization).

is_to_healthCareProvider(A) :-
  is_msg_to_role(A, healthCare_provider).
  %is_to_employeeOf(A, Y),
  %in_role(Y, healthCare_provider).

is_to_secretary(A) :-
  is_msg_to_role(A, government_secretary).

is_to_legalAttorney(A) :-
  is_msg_to_role(A, legal_attorney).

is_to_lawEnforcementOfficer(A) :-
  is_msg_to_role(A, law_enforcement_officer).

is_to_concernedIndividual(A) :-
  is_msg_to_concerned_role(A, individual).

is_to_clergy(A):- 
  is_msg_to_role(A, clergy).

is_to_relative(A):-
  msg_about(A,X), 
  msg_to(A,Y), 
  relationship(Y,relative,A).

is_to_closeFriend(A):-
  msg_about(A,X), 
  msg_to(A,Y), 
  relationship(Y,close_friend,A).

is_to_personIdentified(A):-
  msg_about(A,X), 
  msg_to(A,Y), 
  relationship(Y,person_identified_by,A).

is_to_privateEntity(A):- is_msg_to_role(A,privateEntity).

is_to_publicEntity(A):-is_msg_to_role(A,publicEntity).

is_to_authorizedByLaw_to_assist_disasterRelief(A):-is_msg_to_role(A, authorizedByLaw_to_assist_disasterRelief).

is_to_authorizedByCharter_to_assist_disasterRelief(A):- is_msg_to_role(A,authorizedByCharter_to_assist_disasterRelief).



%%%%%%%%%%% ABOUT
is_about_individual(A) :-
  is_msg_about_role(A, individual).

is_about_suspectedCrimePerpetrator(A) :-
  is_msg_about_role(A, suspected_crime_perpetrator).

is_about_adult(A) :-
  is_msg_about_role(A, adult);
  is_msg_about_role(A, emancipated_minor).

is_about_minor(A) :-
  is_msg_about_role(A, unemancipated_minor).

is_about_deceasedIndividual(A) :-
  is_msg_about_role(A, deceased_individual).

is_about_was_given_consent_opp(A):- 
  msg_about(A,Y),
  is_msg_consent_opp_given(A,Y).


is_msg_consented(a(_,_,_,_,_,_,(X,consented),_),X).

is_msg_consent_opp_given(a(_,_,_,_,_,_,(X,opp_given),_),X). 
% opp_given means opp to decline was given but did not decline



is_about_incapac(A):- 
  is_msg_about_role(A,incapac).

is_about_emerg(A):- 
  is_msg_about_role(A,emerg).

is_about_avail_for_consent(A):- fail.
%possibly add global rules: msg_about(A,Y), curr_state(Y,avail_for_consent).

is_about_present(A):- fail.
%msg_about(A,Y), curr_state(Y,present).

is_about_in_capac_to_make_dec(A):- fail.
%msg_about(A,Y), curr_state(Y, in_capac_to_make_decisions).

is_for_health_record_sale(A) :-
  has_msg_purpose(A, health_record_sale).



%%%%%%%%%% TYPE
is_phi(A) :-
  is_msg_type(A, phi).

is_type_pres_medsupp_xray_etc(A):-
  is_msg_type(A,pres_medsupp_xray_etc).

%%%%%%%%%% PURPOSE
is_for_eitherPurpose(A):-
  has_msg_purpose(A, healthCare_operations);
  %(has_msg_purpose(A, Y),
  %purpose(Y, healthCare_operations));
  has_msg_purpose(A, payment);
  %(has_msg_purpose(A, Y),
  %purpose(Y, payment));
  has_msg_purpose(A, treatment).
  %(has_msg_purpose(A, Y),
  %purpose(Y, treatment)).
  
is_for_incidentToUse(A) :-
  has_msg_purpose(A, incident_to_use).

is_for_createDeidentifiedInfo(A) :-
  has_msg_purpose(A, create_deidentified_info).

is_for_createProtectedHealthInfo(A) :-
  has_msg_purpose(A, create_protected_health_info).

is_for_receiveProtectedHealthInfo(A) :-
  has_msg_purpose(A, receive_protected_health_info).

is_for_investigation(A) :-
   has_msg_purpose(A, investigate).

is_for_treatment(A) :-
   has_msg_purpose(A, treatment).

is_for_payment(A) :-
   has_msg_purpose(A, payment).

is_for_healthCareOperations(A) :-
   has_msg_purpose(A, healthCare_operations).

is_for_definitionOfHealthCareOperations(A) :-
   has_msg_purpose(A, definition_of_healthCare_operations).

is_for_healthCareFraudAbuseDetection(A) :-
   has_msg_purpose(A, healthCare_fraud_abuse_detection).

is_for_compliance(A) :-
   has_msg_purpose(A, compliance).

is_for_determiningLegalOptions(A) :-
   has_msg_purpose(A, determining_legal_options).

is_for_standardsFailureMisconduct(A, Y) :-
   has_msg_purpose(A, standards_failure_misconduct).

is_for_obtainingAuthorization(A):- 
  has_msg_purpose(A, obtaining_authorization).

is_for_directory_purp(A):- has_msg_purpose(A,directory).

is_for_notification_fam_personalrep_respons_of_location(A):-
  has_msg_purpose(A,notification_fam_personalrep_respons_of_location).

is_for_notification_fam_personalrep_respons_of_gencond(A):-
  has_msg_purpose(A,notification_fam_personalrep_respons_of_gencond).

is_for_notification_fam_personalrep_respons_of_death(A):-
  has_msg_purpose(A,notification_fam_personalrep_respons_of_death).

%%%%%%%%%%%%% REPLY TO
is_replyTo_request(A) :-
  is_msg_replyto(A, B),
  B \= null,
  (has_msg_purpose(B, requested_by_Individual);
  is_replyTo_request(B)).

%%%%%%%%%%%%% CONSENTED BY
is_consentedby_about(A) :-
  msg_about(A, X),
  is_msg_consented(A,X).

%%%%%%%%%%%%% BELIEF
is_belief_from_lawfulBusinessAssociate(A, Y) :-
  msg_from(A, X),
  has_msg_belief(A, Y, lawful_business_associate, X).

is_belief_to_lawfulBusinessAssociate(A, Y) :-
  msg_to(A, X),
  has_msg_belief(A, Y, lawful_business_associate, X).

is_belief_from_unlawfulCoveredEntity(A, Y) :-
  msg_from(A, X),
  has_msg_belief(A, Y, unlawful_covered_entity, X).

is_belief_from_employeeVictimOfCriminalAct(A, Y) :-
  msg_from(A, X),
  has_msg_belief(A, Y, employee_victim_of_criminal_act, X).

is_belief_from_minimum(A):- 
  msg_from(A, X),
  has_msg_belief(A, _, minimum_necessary_to_purpose, X).

is_belief_from_about_pertainingToRelationship(A):- 
  msg_from(A, X),
  msg_about(A,Y),
  has_msg_belief(A, Y, minimum_necessary_to_purpose, X).

is_belief_best_interest(A):-
msg_from(A, X),
msg_about(A,Y),
has_msg_belief(A, Y, in_indivs_best_interest, X).
%  is_msg_belief(A,in_indivs_best_interest).

is_belief_not_disclosing_would_interfere_with_emergResponse(A):-
msg_from(A, X),
has_msg_belief(A, _,not_disclosing_would_interfere_with_emergResponse, X).
%  is_msg_belief(A,not_disclosing_would_interfere_with_emergResponse).

is_belief_can_be_inferred_indiv_wouldnt_object(A):-
msg_from(A, X),
msg_about(A,Y),
has_msg_belief(A, Y, can_be_inferred_indiv_wouldnt_object, X).
%  is_msg_belief(A,can_be_inferred_indiv_wouldnt_object).



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
is_nam_loc_or_condition(A):- 
  is_msg_type(A,name); 
  is_msg_type(A,loc); 
  is_msg_type(A,cond).

is_type_relig(A):- 
  is_msg_type(A,relig).

is_relevant_to_payment_or_health_involvement(A):-
 msg_to(A,X), 
 msg_about(A,Y),
 ((involved(X,med_care,Y), 
   has_msg_purpose(A,med_care));
  (involved(X,payment,Y),
   has_msg_purpose(A,payment))).




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%% in_role heirarchy
in_role(healthCare_provider, covered_entity).
in_role(healthCare_plan, covered_entity).
in_role(healthCare_clearing_house, covered_entity).
in_role(X,Y) :-
  inRole(X,Y).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%% type heirarchy
in_type(name, phi).
in_type(loc,phi).
in_type(cond,phi).
in_type(relig,phi).
in_tyoe(pres_medsupp_xray_etc,phi).


in_type_closures(X,X).

in_type_closures(X, Y) :-
  in_type(X, Y).

in_type_closures(X, Y) :-
  in_type(X, Z),
  in_type_closures(Z, Y).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%transitive closure of basic relations.
%%The terminal relation should be specified outside.

%Transitive closures are going into infinite loops

in_role_closures(X, Y) :-
  in_role(X, Y).

in_role_closures(X, Y) :-
  in_role(X, Z),
  in_role_closures(Z, Y).

%in_role(X, Y) :-
  %in_role(X, Z),
  %in_role(Z, Y).

%in_relation(X, Y) :-
  %in_relation(X, Z),
  %in_relation(Z, Y).


%%%%%%%%%%%%%%%%%MISC

is_minimum_necessary(A) :-
  is_msg_minimum_necessary(A, yes).
% Make sure that the file including odbc.pl has loaded the odbc_call
% with :- [odbc_call].
:- odbc_open('MySQL-test', root, user). 

% assert_in_role(MySQL_tuples)
%
%  Pulls inRole rules from SQL Database and adds them  to the Prolog Database
assert_in_role(_) :-
	findall(inRole(Person, Role), odbc_sql([], 'SELECT person, role FROM in_role', [Person, Role]), Results),
	assert_each(Results).

% find_role_leaves(Leaves)
%
%  Retrieves all terminating roles
find_role_leaves(Leaves) :-
	findall(Person, odbc_sql([], 'SELECT DISTINCT person FROM in_role', [Person]), Results),
	filter_leaves(Results, Leaves).

% assert_each(Rule_list)
%   Adds each rule from the Rule_list into Prolog Database
assert_each([]).    
assert_each([H|T]) :-
%	writeln(H),
	assert(H),
	assert_each(T).

% filter_leaves(Role_nodes, Leaves)
%
%   Given a list of role nodes, returns a filtered list of leaves
filter_leaves([], []).
filter_leaves([H|T], [X|Y]) :-
	atom(H),
	is_role_leaf(H),
	H=X,
	filter_leaves(T,Y).
filter_leaves([_|T], L) :-
	filter_leaves(T, L).


%%% HIPAA
%%%%%%%%%%%%%%%%%%%%%%%% 
%%Roles
%------
%healthCare_plan
%healthCare_clearing_house
%healthCare_provider
%health_oversight_agency
%public_health_authority
%healthCare_accreditation_organization
%coveredEntity
%secretary
%legal_attorney
%law_enforcement_officer
%suspected_crime_perpetrator
%individual - means patient at that hospital
%adult
%emancipated_minor
%unemancipated_minor
%deceased_individual
%healthcare_provider
%health_plan
%health_insurance_issuer
%group_health_plan
%government_agency_health_plan

%% msg types
%-----------
%phi

%%Purpose
%--------
%healthCare_operations
%payment
%treatment
%incident_to_use
%create_deidentified_info
%create_protected_health_info
%receive_deidentified_info
%requested_by_Individual
%investigate
%definition_healthCare_operations
%healthCare_fraud_abuse_detection
%compliance
%determining_legal_options
%standards_failure_misconduct

%% Basic relations
%-----------------
% in_role(X, Y) says if X is in role Y. Here mention if covered entity.
% employee_of(X, Y) says if X is employee of Y
% in_relation(X, Y) says if X is related to Y (its not commutative) X gives rights to Y
% minimum(X, Y) says if for type of message X and purpose Y its minimum information
% business_associate(X, Y) says if X is a business associate of Y.
% personal_representative(X, Y) says if Y is a personal representative of X, includes executor, administrator by law.
% parent(X, Y), guardian(X, Y), loco_parentis(X, Y) says if y is parent of X

%% There is just this one Covered Entity!

%%permitted_by_HIPAA(A):-

pbh(A):-
  cwh(A).
  %per(A).

per(A):-
  (permitted_by_164_502(A);
   permitted_by_164_506(A);
   permitted_by_164_510(A);
  %permitted_by_self(A);
  fail).

cwh(A):-
  (permitted_by_164_502(A);
   permitted_by_164_506(A);
   permitted_by_164_510(A);
   permitted_by_13405_e(A);
  %permitted_by_self(A);
  fail),
  \+ (forbidden_by_164_502(A);
   forbidden_by_164_506(A);
   forbidden_by_164_510(A);
   forbidden_by_13405_d(A)).

permitted_by_self(A) :- 
  msg_from(A, X),
  msg_about(A, Y),
  X = Y.
%Uses and disclosures for the facilty directory

permitted_by_164_510_a(A) :-
permitted_by_164_510_a_1(A),
(permitted_by_164_510_a_2(A) ; excluded_164_510_a_3(A)).

permitted_by_164_502_a_1(A):- 
(is_for_directory_purp(A), is_nam_loc_or_condition(A),
%need to implement "asked by name"
 fail); 
(is_to_clergy(A),(is_type_relig(A);is_nam_loc_or_condition(A))).

permitted_by_160_510_a_2(A):- is_about_was_given_consent_opp(A).

excluded_by_164_510_a_3(A):- 
(is_about_incapac(A);is_about_emerg(A)),
fail,
% not sure how to implement is consistent with past, 
 is_belief_best_interest(A).

%Uses and disclosures for involvement in the individual's care and notification purposes 

permitted_by_164_510_b(A) :-
(permitted_by_164_510_b_1(A),
 permitted_by_164_510_b_2(A),
 permitted_by_164_510_b_3(A));
permitted_by_164_510_b_4(A).

permitted_by_164_510_b_1(A) :-
permitted_by_164_510_b_1_i(A);
permitted_by_164_510_b_1_ii(A).

permitted_by_164_510_b_1_i(A):- 
is_phi(A), 
is_from_coveredEntity(A), 
(is_to_relative(A);is_to_closeFriend(A);is_to_personIdentified(A)), 
is_relevant_to_payment_or_health_involvement(A).


permitted_by_164_510_b_1_ii(A):- 
is_phi(A),
is_from_coveredEntity(A), 
(is_for_notification_fam_personalrep_respons_of_location(A);
is_for_notification_fam_personalrep_respons_of_gencond(A);
is_for_notification_fam_personalrep_respons_of_death(A)).

%not sure how to implement these yet:
%is_about_present,is_about_avail_for_consent,is_about_in_capac_to_make_dec
%currently they fail

permitted_by_164_510_b_2(A):-
((not(is_about_present(A)),not(is_about_avail_for_consent(A)));
(is_about_in_capac_to_make_dec(A));
(is_consentedby_about(A);
(is_about_was_given_consent_opp(A), is_msg_consented(A));
is_belief_best_interest(A)).

permitted_by_164_510_b_3(A):- 
(is_about_present(A));
(not(is_about_incapac(A)),not(is_about_emerg(A)));
(is_belief_best_interest(A), 
(is_relevant_to_payment_or_health_involvement(A);
is_msg_type(A,prescription))).

permitted_by_164_510_b_4(A): 
permitted_by_164_510_b_1_ii(A), 
(is_to_privateEntity(A);is_to_publicEntity(A)), 
(is_to_authorizedByLaw_to_assist_disasterRelief(A);is_to_authorizedByCharter_to_assist_disasterRelief(A)),
((permitted_by_164_510_b_2(A),permitted_by_164_510_b_3(A));is_belief_not_disclosing_would_interfere_with_emergResponse(A)).

%% Uses and disclosures requiring an opportunity for the individual to agree or 
to object

permitted_by_164_510(A)
is_about_was_given_consent_opp(A),
is_phi(A),
(permitted_by_164_510_a(A); permitted_by_164_510_b(A).

%%Compliance and Investigations
permitted_by_160_C(A) :-
  debug('160.C: not implemeted compliance and investigations').


%%Standard rules for "uses and disclosures"
permitted_by_164_502_a(A) :-
  is_from_coveredEntity(A),
  is_phi(A),
  (permitted_by_160_C(A);
  permitted_by_164_502_a_1(A);
  required_by_164_502_a_2(A)).

permitted_by_164_502_a_1(A):-                
  permitted_by_164_502_a_1_i(A);
  permitted_by_164_502_a_1_ii(A);
  permitted_by_164_502_a_1_iii(A);
  permitted_by_164_502_a_1_iv(A);
  permitted_by_164_502_a_1_v(A);
  permitted_by_164_502_a_1_vi(A).

permitted_by_164_502_a_1_i(A):-
  (is_to_concernedIndividual(A);
  is_from_concernedIndividual(A)),
  writeln('HIPAA rule 164_502_a_1_i;').

permitted_by_164_502_a_1_ii(A):-
  is_for_eitherPurpose(A),
  permitted_by_164_506(A),              
  writeln('HIPAA rule 164_502_a_1_ii;').

permitted_by_164_502_a_1_iii(A):-
  is_for_incidentToUse(A),
  permitted_by_164_502_b(A),   
  permitted_by_164_514_d(A),   
  permitted_by_164_530_c(A),              
  writeln('HIPAA rule 164_502_a_1_iii;').

permitted_by_164_502_a_1_iv(A):-
  require_authorization_by_164_508(A),
  writeln('HIPAA rule 164_502_a_1_iv;').

permitted_by_164_502_a_1_v(A):-
  permitted_by_164_510(A),                  
  writeln('HIPAA rule 164_502_a_1_v;').

permitted_by_164_502_a_1_vi(A):-
  permitted_by_164_512(A);
  permitted_by_164_514_e(A);
  permitted_by_164_514_f(A);
  permitted_by_164_514_g(A).

%required by!! 
required_by_164_502_a_2(A):-   
  required_by_164_502_a_2_i(A);
  required_by_164_502_a_2_ii(A).

required_by_164_502_a_2_i(A):-
  is_to_concernedIndividual(A),
  is_replyTo_request(A),
  (required_by_164_524(A);
  required_by_164_528(A)),
  writeln('HIPAA rule 164_502_a_2_i;').

required_by_164_502_a_2_ii(A):-
   is_to_secretary(A),
   is_for_investigation(A),
   permitted_by_160_C(A),
   writeln('HIPAA rule 164_502_a_2_ii;').

forbidden_by_164_502_a(A) :-
 fail.

%%Standard: Minimum neccessary.
%Must be minimum information or law does not apply
permitted_by_164_502_b(A) :-
  permitted_by_164_502_b_1(A);
  excluded_164_502_b_2(A).
  
permitted_by_164_502_b_1(A) :-
  is_from_coveredEntity(A),
  is_to_coveredEntity(A),
  is_belief_from_minimum(A),
  writeln('HIPAA rule 164_502_b_1;').
  
%Minimum necessary does not apply in these cases.
excluded_164_502_b_2(A) :-
  excluded_164_502_b_2_i(A);
  excluded_164_502_b_2_ii(A);
  excluded_164_502_b_2_iii(A);
  excluded_164_502_b_2_iv(A);
  excluded_164_502_b_2_v(A);
  excluded_164_502_b_2_vi(A).  
  
excluded_164_502_b_2_i(A) :-
  is_for_treatment(A),
  is_to_healthCareProvider(A),
  writeln('HIPAA rule 164_502_b_2_i;').
  
excluded_164_502_b_2_ii(A) :-
  (permitted_by_164_502_a_1_i(A);
  required_by_164_502_a_2_i(A)),
  writeln('HIPAA rule 164_502_b_2_ii;').
  
excluded_164_502_b_2_iii(A) :-
  is_for_obtainingAuthorization(A),
  writeln('HIPAA rule 164_502_b_2_iii;').
  
excluded_164_502_b_2_iv(A) :-
  is_to_secretary(A),
  permitted_by_160_C(A),
  writeln('HIPAA rule 164_502_b_2_iv;').
   
excluded_164_502_b_2_v(A) :-   
  required_by_164_512_a(A),
  writeln('HIPAA rule 164_502_b_2_v;').
   
%Need to add all required types. Found only this.	
excluded_164_502_b_2_vi(A) :-
  required_by_164_502_a_2(A),
  writeln('HIPAA rule 164_502_b_2_vi;').

forbidden_by_164_502_b(A) :-
  fail. 
  


%%Uses and Disclosure of protected health information subject to agreed upon restriction

permitted_by_164_502_c(A) :-
% must also check whether restriction exists for a particular case 
  is_from_coveredEntity(A),
  is_phi(A),
  (permitted_by_164_522_a_1(A);
   permitted_by_164_522_a(A)),
  writeln('HIPAA rule 164_502_c;').

forbidden_by_164_502_c(A) :-
  fail.

%%Uses and disclosure of deindentified information

permitted_by_164_502_d(A) :-
  permitted_by_164_502_d_1(A);
  permitted_by_164_502_d_2(A).

permitted_by_164_502_d_1(A) :-
%use part is not so clear, just modelled the disclosure part.
  is_phi(A),
  is_from_coveredEntity(A),
  is_to_businessAssociateOf(A),
  is_for_createDeidentifiedInfo(A),
  writeln('HIPAA rule 164_502_d_1;').

permitted_by_164_502_d_2(A) :-
  permitted_by_164_514(A),
  (excluded_by_164_502_d_2_i(A);
  permitted_by_164_502_d_2_ii(A)),
  writeln('HIPAA rule 164_502_b_2;').

excluded_by_164_502_d_2_i(A) :-
%verify that the message does not have identifiable attributes for join like primary keys
  fail.

permitted_by_164_502_d_2_ii(A) :-
% verify that the de-indentified information is not re-identified 
  fail.

forbidden_by_164_502_d(A) :-
 fail.

%%Disclosures to Business Associates

permitted_by_164_502_e(A):-
  excluded_164_502_e_1_ii(A);
  permitted_by_164_502_e_1_i(A).
  %not sure what this means. non-compliance of BA.
  %permitted_164_502_e_1_iii(A).
  %surely cant implement this written notice thing.
  %permitted_164_502_e_2(A).

%See if the message is allowed to be received by any of the covered entities of 
%this business associate or if the covered entity is disclosing protected health.
%to a business associate. An entity is a business associate of some covered entity 
%if that covered entity receives assurance that the BA will appropriately
%safeguard the info and act on behalf of the covered entity. 

permitted_by_164_502_e_1_i(A) :-
  is_phi(A),
  ( (is_from_coveredEntity(A),
     is_to_businessAssociateOf(A),
     is_belief_to_lawfulBusinessAssociate(A, X));
    (is_to_coveredEntity(A),
     is_from_businessAssociateOf(A),
     is_belief_from_lawfulBusinessAssociate(A, X))
  ),
  (is_for_createProtectedHealthInfo(A);
   is_for_receiveProtectedHealthInfo(A)),
  writeln('HIPAA rule 164_502_e_1_i;').

excluded_164_502_e_1_ii(A):-
  excluded_164_502_e_1_ii_a(A);
  excluded_164_502_e_1_ii_b(A);
  excluded_164_502_e_1_ii_c(A).
  
excluded_164_502_e_1_ii_a(A) :-
  is_about_individual(A),
  is_to_healthCareProvider(A),
  is_for_treatment(A),
  is_from_coveredEntity(A),
  writeln('HIPAA rule 164_502_e_1_ii_a;').
  
excluded_164_502_e_1_ii_b(A) :-
  %to plan sponsor
  (is_from_healthInsuranceIssuer(A);
   is_from_groupHealthPlan(A)),
  permitted_by_164_504_f(A),
  writeln('HIPAA rule 164_502_e_1_ii_b;').
% "with respect to a group health plan" could be represented by storing group health plan in type or purpose? 
  
%enrollment information is collected by the government agency.  
%(incomplete) lot of insurance stuff
excluded_164_502_e_1_ii_c(A) :- 
  is_from_governmentAgencyHealthPlan(A),
  writeln('HIPAA rule 164_502_e_1_ii_c;').

forbidden_by_164_502_e(A) :-
  fail.

%%Uses and disclosure of deceased individual

permitted_by_164_502_f(A) :-
%comply to subpart if individual dead
% so basically dont care dead or alive.
  fail.

forbidden_by_164_502_f(A) :-
  fail.

%%Uses and disclosure to personal representatives

permitted_by_164_502_g(A) :-
  permitted_by_164_502_g_1(A);
  permitted_by_164_502_g_2(A);
  permitted_by_164_502_g_3(A);
  permitted_by_164_502_g_4(A);
  permitted_by_164_502_g_5(A).

permitted_by_164_502_g_1(A) :-
%treat personal representative as individual except by g-3 ang g-5.
  fail,
  writeln('HIPAA rule 164_502_g_1;').

permitted_by_164_502_g_2(A) :-
% should treat adult or emancipated minor in making decision as individual, 
% relavant to such personal representatives.
  is_about_adult(A),
  msg_about(A, X),
  personal_representative(X, Y),
  (msg_to(A, Y);
  msg_from(A, Y)),
  writeln('HIPAA rule 164_502_g_2;').

is_guardian(X, Y) :-
  parent(X, Y);
  guardian(X, Y);
  loco_parentis(X, Y).

permitted_by_164_502_g_3(A) :-
%Assuming there are no applicable provisions or other applicable state laws,
%should be allowed if 3i and 164.524 allow it, and denied if they deny.
% else (when neither are applicable) the decision is made by licensed health professional

  permitted_by_164_502_g_3_i(A);
  permitted_by_164_502_g_3_ii(A).



permitted_by_164_502_g_3_i(A) :-
% relavant to such personal representatives.
% about health care services
  is_about_minor(A),
  msg_about(A, X),
  (msg_to(A, Y);
  msg_from(A,Y)),
  is_guardian(X, Y),
  \+ (permitted_by_164_502_g_3_i_A(A),
      permitted_by_164_502_g_3_i_B(A),
      permitted_by_164_502_g_3_i_C(A)),
  writeln('HIPAA rule 164_502_g_3_i;').
%rule requires you to remove a parents role
% as a personal representative globally in the shh file. 
% this can be done if the child requests such or parent agrees 

permitted_by_164_502_g_3_i_A(A) :-
  fail.
  
permitted_by_164_502_g_3_i_B(A) :-
  fail.

permitted_by_164_502_g_3_i_C(A) :-
  fail.

permitted_by_164_502_g_3_ii(A) :-
  permitted_by_164_502_g_3_ii_A(A);
  permitted_by_164_502_g_3_ii_B(A);
  permitted_by_164_502_g_3_ii_C(A).

permitted_by_164_502_g_3_ii_A(A) :-
% covered entity may send to guardian based on applicable law
  fail,
  writeln('HIPAA rule 164_502_g_3_ii_A;').
%add if permitted by any othe sepcified by laws

permitted_by_164_502_g_3_ii_B(A) :-
% covered entity may NOT send to guardian based on applicable law
  fail,
  writeln('HIPAA rule 164_502_g_3_ii_B;').

permitted_by_164_502_g_3_ii_C(A) :-
% will add one more rule representing personal representative in add. to guardian
%if personal rep ties are broken, then
% licensed medical practioner may send based on professional judgement
  fail,
  writeln('HIPAA rule 164_502_g_3_ii_C;').

permitted_by_164_502_g_4(A) :-
  is_about_deceasedIndividual(A),
  msg_about(A, X),
  personal_representative(X, Y),
  (msg_to(A, Y);
  msg_from(A, Y)),
  writeln('HIPAA rule 164_502_g_4;').
  
permitted_by_164_502_g_5(A) :-
  permitted_by_164_502_g_5_i(A);
  permitted_by_164_502_g_5_ii(A).

permitted_by_164_502_g_5_i(A) :-
  msg_about(A, X),
  msg_to(A, Y),
  %is_belief_dangerousRepresentative(X, Y).
  fail,
  writeln('HIPAA rule 164_502_g_5_i;').

permitted_by_164_502_g_5_ii(A) :-
  %is_belief_notBestInterestOfIndividual(X, Y).
  fail,
  writeln('HIPAA rule 164_502_g_5_ii;').

forbidden_by_164_502_g(A) :-
  fail.



%%Confidential communication

permitted_by_164_502_h(A) :-
  (is_from_healthCareProvider(A);
  is_from_healthPlan(A)),
  is_phi(A),
  permitted_by_164_522_b(A),
  writeln('HIPAA rule 164_502_h;').


forbidden_by_164_502_h(A) :-
  fail.


%%Uses and disclosure consistent with notice
% ensure message exchange is consistent with notice
permitted_by_164_502_i(A) :-
  debug('160._502_i: not implemeted disclosure with notice;').

forbidden_by_164_502_i(A) :-
  fail.

%%Disclosure by whistleblowers and work force members crime victims

permitted_by_164_502_j(A) :-
  permitted_by_164_502_j_1(A);
  permitted_by_164_502_j_2(A).

permitted_by_164_502_j_1(A) :-
   (is_from_employeeOf(A, Y);
    is_from_businessAssociateOf(A, Y)),
   permitted_by_164_502_j_1_i(A, Y),
   permitted_by_164_502_j_1_ii(A, Y).

permitted_by_164_502_j_1_i(A, Y) :-
%basically this belief includes "they believe in good faith that the covered Entity has engaged in conduct
% that is unlawful or otherwise violates professional or clinical standards, or that the care, 
% services or conditions provided by covered entity potentially endangers one or more patients, 
% workers or the public"
  is_belief_from_unlawfulCoveredEntity(A, Y),
  writeln('HIPAA rule 164_502_j_1_i;').

permitted_by_164_502_j_1_ii(A, Y) :-
  permitted_by_164_502_j_1_ii_A(A, Y);
  permitted_by_164_502_j_1_ii_B(A, Y).

permitted_by_164_502_j_1_ii_A(A, Y) :-
  is_to_healthOversightAgency(A);

  (is_to_publicHealthAuthority(A),
  is_for_investigation(A));

  (is_to_healthCareAccreditationOrganization(A),
  is_for_standardsFailureMisconduct(A, Y)),

  writeln('HIPAA rule 164_502_j_1_ii_A;').

permitted_by_164_502_j_1_ii_B(A, Y) :-
  is_to_legalAttorney(A),
  is_for_determiningLegalOptions(A),
  %is_belief_from_unlawfulCoveredEntity(A, Y),
  writeln('HIPAA rule 164_502_j_1_ii_B;').

permitted_by_164_502_j_2(A) :-
  is_from_employeeOf(A, Y),
  is_belief_from_employeeVictimOfCriminalAct(A, Y),
  is_to_lawEnforcementOfficer(A),
  is_phi(A),
  permitted_by_164_502_j_2_i(A),
  permitted_by_164_502_j_2_ii(A),
  writeln('HIPAA rule 164_502_j_2;').
  
permitted_by_164_502_j_2_i(A) :-
  is_about_suspectedCrimePerpetrator(A).

permitted_by_164_502_j_2_ii(A) :-
  permitted_by_164_512_f_2_i(A).

forbidden_by_164_502_j(A) :-
  fail.

%uses and disclosure of PHI


permitted_by_164_502(A):-                
  permitted_by_164_502_a(A);
  permitted_by_164_502_b(A);	%must satisfy
  permitted_by_164_502_c(A);
  permitted_by_164_502_d(A);
  permitted_by_164_502_e(A);	%must satisfy
  permitted_by_164_502_f(A);
  permitted_by_164_502_g(A);
  permitted_by_164_502_h(A);
  permitted_by_164_502_i(A);
  permitted_by_164_502_j(A).

forbidden_by_164_502(A):-                
  forbidden_by_164_502_a(A);
  forbidden_by_164_502_b(A);
  forbidden_by_164_502_c(A);
  forbidden_by_164_502_d(A);
  forbidden_by_164_502_e(A);
  forbidden_by_164_502_f(A);
  forbidden_by_164_502_g(A);
  forbidden_by_164_502_h(A);
  forbidden_by_164_502_i(A);
  forbidden_by_164_502_j(A).


%% Uses and disclosures for which authorization is required.
  %should check if all valid authorizations have been obtained
permitted_by_164_504_f(A) :-
  debug('164.504: TBD ;').
 


%% Uses and disclosures to carry out treatment, payment or health care operations.
permitted_by_164_506(A) :-
  permitted_by_164_506_a(A);
  permitted_by_164_506_b(A);
  permitted_by_164_506_c(A).

permitted_by_164_506_a(A) :-
%Standard permitted uses and disclosures
  \+ require_authorization_by_164_508(A),
  is_from_coveredEntity(A),
  is_for_eitherPurpose(A),
  permitted_by_164_506_c(A),
  writeln('HIPAA rule 164_506_a;').
  
permitted_by_164_506_b(A) :-
% This consent is only sufficient if authorization is not 
% required or there is no other condition before the diclosure 
% that needs to be met. 
  is_for_eitherPurpose(A),
  is_consentedby_about(A).  

permitted_by_164_506_c(A) :-
%Implementation Specification
  permitted_by_164_506_c_1(A);
  permitted_by_164_506_c_2(A);
  permitted_by_164_506_c_3(A);
  permitted_by_164_506_c_4(A);
  permitted_by_164_506_c_5(A).

permitted_by_164_506_c_1(A) :-
  is_for_eitherPurpose(A),
  is_phi(A),
  is_msg_to_within(A).

permitted_by_164_506_c_2(A) :-
  is_for_treatment(A),
  is_phi(A),
  is_to_healthCareProvider(A).

permitted_by_164_506_c_3(A) :-
  is_for_payment(A),
  is_phi(A),
  (is_to_healthCareProvider(A);
  is_to_coveredEntity(A)).

permitted_by_164_506_c_4(A) :-
  %writeln('HIPAA rule 164.506.c.4: How to ensure that its a diff covered entitiy? information pertains to that relation'),
  is_from_coveredEntity(A),
  is_to_coveredEntity(A), 
  is_for_healthCareOperations(A),
  %pertains_to(A),
  is_belief_from_about_pertainingToRelationship(A),
  is_msg_about_to_inRelation(A).
  %satisfy_164_506_c_4(A).

satisfy_164_506_c_4(A) :-
  %%Pupose has to be helath care operations along with the ones below.
  %%Currently not working as you need a list of purposes. Right now its just one purpose.
  permitted_by_164_506_c_4_i(A);
  permitted_by_164_506_c_4_ii(A).

permitted_by_164_506_c_4_i(A) :-
  debug('164.506.c.4.i: Could not figure out this para;'),
  is_for_definitionOfHealthCareOperations(A).

permitted_by_164_506_c_4_ii(A) :-
  is_for_healthCareFraudAbuseDetection(A);
  is_for_compliance(A).
  
% Create organizations for each organization and have global rules that
% link each member to the organization. Then just confirm that from and to
% are part of same organization and the purpose is the organization's
%purpose
permitted_by_164_506_c_5(A) :-
  debug('164.506_c_5: Not Implemented yet;').

forbidden_by_164_506(A) :-
  fail.

%% Uses and disclosures for which authorization is required.
  %should check if all valid authorizations have been obtained
require_authorization_by_164_508(A) :-
  debug('164.508: TBD no authorizations required, returns true;').
 
%Uses and disclosures for the facilty directory

permitted_by_164_510_a(A) :-
permitted_by_164_510_a_1(A),
(permitted_by_164_510_a_2(A) ; excluded_by_164_510_a_3(A)).

permitted_by_164_510_a_1(A):- 
(is_for_directory_purp(A), is_nam_loc_or_condition(A),
%need to implement "asked by name"
 fail); 
(is_to_clergy(A),(is_type_relig(A);is_nam_loc_or_condition(A)),writeln('HIPAA rule 164_510_a_1')).

permitted_by_164_510_a_2(A):- 
is_about_was_given_consent_opp(A),writeln('HIPAA rule 164_510_a_2').

excluded_by_164_510_a_3:- 
(is_about_incapac(A);is_about_emerg(A)),
fail,
% not sure how to implement is consistent with past, 
 is_belief_best_interest(A),writeln('HIPAA rule 164_510_a3').%Uses and disclosures for involvement in the individual's care and notification purposes 


permitted_by_164_510_b(A) :-
(permitted_by_164_510_b_1(A),
 permitted_by_164_510_b_2(A),
 permitted_by_164_510_b_3(A));
permitted_by_164_510_b_4(A).

permitted_by_164_510_b_1(A) :-
permitted_by_164_510_b_1_i(A);
permitted_by_164_510_b_1_ii(A).

permitted_by_164_510_b_1_i(A):- 

is_phi(A), 
is_from_coveredEntity(A), 
(is_to_relative(A);is_to_closeFriend(A);is_to_personIdentified(A)), 
is_relevant_to_payment_or_care_involvement(A),writeln('HIPAA rule 164_510_b_1_i').


permitted_by_164_510_b_1_ii(A):- 
is_phi(A),
is_from_coveredEntity(A), 
(is_for_notification_fam_personalrep_respons_of_location(A);
is_for_notification_fam_personalrep_respons_of_gencond(A);
is_for_notification_fam_personalrep_respons_of_death(A)),
writeln('HIPAA rule 164_510_b_1_ii;').

%not sure how to implement these yet:
%is_about_present,is_about_avail_for_consent,is_about_in_capac_to_make_dec
%currently they fail

permitted_by_164_510_b_2(A):-
((not(is_about_present(A));not(is_about_avail_for_consent(A))),
not(is_about_in_capac_to_make_dec(A)));
((is_about_present(A);is_about_avail_for_consent(A)),
(is_about_in_capac_to_make_dec(A)),
(is_consentedby_about(A);
 is_about_was_given_consent_opp(A);
 is_belief_can_be_inferred_indiv_wouldnt_object(A))),
 writeln('HIPAA rule 164_510_b_2').

%permitted_by_164_510_b_2(A):-
%(is_about_present(A);is_about_avail_for_consent(A)),
%(is_about_in_capac_to_make_dec(A)),
%(is_consentedby_about(A);
 %is_about_was_given_consent_opp(A);
 %is_belief_can_be_inferred_indiv_wouldnt_object(A)),
% writeln('HIPAA rule 164_510_b_2').

permitted_by_164_510_b_3(A):- 
(is_about_present(A),
(not(is_about_incapac(A)),not(is_about_emerg(A))));
(is_belief_best_interest(A), 
(is_relevant_to_payment_or_health_involvement(A);
is_msg_type(A,pres_medsupp_xray_etc))),
writeln('HIPAA rule 164_510_b_3').

%permitted_by_164_510_b_3(A):- 
%(is_about_present(A));
%(not(is_about_incapac(A)),not(is_about_emerg%%(A)));
%(is_belief_best_interest(A), 
%(is_relevant_to_payment_or_care_involvement(A);
%is_msg_type(A,prescription))),
%writeln('HIPAA rule 164_510_b_3').

permitted_by_164_510_b_4(A):- 
permitted_by_164_510_b_1_ii(A), 
(is_to_privateEntity(A);is_to_publicEntity(A)), 
(is_to_authorizedByLaw_to_assist_disasterRelief(A);is_to_authorizedByCharter_to_assist_disasterRelief(A)),
((permitted_by_164_510_b_2(A),permitted_by_164_510_b_3(A));is_belief_not_disclosing_would_interfere_with_emergResponse(A)),
writeln('HIPAA rule 164_510_b_4').
	

%% Uses and disclosures requiring an opportunity for the individual to agree or to object

permitted_by_164_510(A):-
  is_phi(A),
  (permitted_by_164_510_a(A); permitted_by_164_510_b(A)).

forbidden_by_164_510(A):-
  fail.

permitted_by_164_512(A) :-
  debug('164.512: not implemeted uses and disclosure where authorization not required;').

required_by_164_512_a(A) :-
  debug('164.512_a: NI: (a) Standard: Uses and disclosures required by law;').

permitted_by_164_512_f_2_i(A) :-
  debug('164.512_f_2_i: NI:  Standard: Disclosures for law enforcement, Permitted disclosures: Limited information for identification and location purposes;').


permitted_by_164_514(A) :-
  debug('164.514: NI: Other requirements relating to uses and disclosures of protected health information;').

permitted_by_164_514_d(A) :-
  debug('164.514_e: NI: Other requirements relating to uses and disclosures of protected health information;').  

permitted_by_164_514_e(A) :-
  debug('164.514_e: NI: Other requirements relating to uses and disclosures of protected health information;').  
  
permitted_by_164_514_f(A) :-
  debug('164.514_f: NI: Other requirements relating to uses and disclosures of protected health information;').  
  
permitted_by_164_514_g(A) :-
  debug('164.514_f: NI: Other requirements relating to uses and disclosures of protected health information;').  

permitted_by_164_522_a(A) :-
  debug('164.522_a: NI: Rights to request privacy protection of protected health information;').

permitted_by_164_522_a_1(A) :-
  debug('164.522_a_1: NI: Rights to request privacy protection of protected health information;').  

permitted_by_164_522_b(A) :- 
  debug('164.522_b: NI: Rights to request privacy protection of protected health information;').  
  


required_by_164_524(X) :-
  debug('164.524; Access of individuals to protected health information, returns fail;').
required_by_164_528(X) :-
  debug('164.528; Accounting of disclosures of protected health information, returns fail;').

permitted_by_164_530_c(A) :-
  debug('164.530: NI: Administrative requirements;').


%%% HIPAA
%%%%%%%%%%%%%%%%%%%%%%%% 
%%Roles
%------
%healthCare_plan
%healthCare_clearing_house
%healthCare_provider
%health_oversight_agency
%public_health_authority
%healthCare_accreditation_organization
%coveredEntity
%secretary
%legal_attorney
%law_enforcement_officer
%suspected_crime_perpetrator
%individual - means patient at that hospital
%adult
%emancipated_minor
%unemancipated_minor
%deceased_individual
%healthcare_provider
%health_plan
%health_insurance_issuer
%group_health_plan
%government_agency_health_plan

%% msg types
%-----------
%phi

%%Purpose
%--------
%healthCare_operations
%payment
%treatment
%incident_to_use
%create_deidentified_info
%create_protected_health_info
%receive_deidentified_info
%requested_by_Individual
%investigate
%definition_healthCare_operations
%healthCare_fraud_abuse_detection
%compliance
%determining_legal_options
%standards_failure_misconduct

%% Basic relations
%-----------------
% in_role(X, Y) says if X is in role Y. Here mention if covered entity.
% employee_of(X, Y) says if X is employee of Y
% in_relation(X, Y) says if X is related to Y (its not commutative) X gives rights to Y
% minimum(X, Y) says if for type of message X and purpose Y its minimum information
% business_associate(X, Y) says if X is a business associate of Y.
% personal_representative(X, Y) says if Y is a personal representative of X, includes executor, administrator by law.
% parent(X, Y), guardian(X, Y), loco_parentis(X, Y) says if y is parent of X

%% There is just this one Covered Entity!

%%permitted_by_HIPAA(A):-




permitted_by_self(A) :- 
  msg_from(A, X),
  msg_about(A, Y),
  X = Y.


%Remove writeln when we don't want to see messages.  Set writeln to an AND of true or fail.  
%True if we want to assume unimplemented portions of code pass.
%Fail if we want to assume unimplemented portions of code always fail.
debug(X) :-
  fail.
  %writeln(X).
%%Example system
:- ['pbh.pl'].
:- ['myfunc.pl'].
inRole(sacred_heart_hospital, covered_entity).
inRole(business_associate, covered_entity).

inRole(chief_of_medicine, healthCare_provider).
inRole(doctor, healthCare_provider).
inRole(surgeon, healthCare_provider).
inRole(intern, healthCare_provider).
inRole(nurse, healthCare_provider).
inRole(volunteer, healthCare_provider).
inRole(trainee, healthCare_provider).

inRole(dr_cox, doctor).
inRole(dr_kelso, chief_of_medicine).
inRole(dr_turk, surgeon).
inRole(dr_jd, intern).
inRole(dr_elliot, intern).
inRole(carla, nurse).
inRole(lavern, secretary).
inRole(ted, legal_attorney).
inRole(j, janitor).
inRole(jordon, board_member).

inRole(dr_cox, adult).
inRole(dr_cox, individual).
inRole(dr_kelso, adult).
inRole(dr_kelso, individual).
inRole(dr_turk, adult).
inRole(dr_turk, individual).
inRole(dr_jd, adult).
inRole(dr_jd, individual).
inRole(dr_elliot, adult).
inRole(dr_elliot, individual).
inRole(carla, adult).
inRole(carla, individual).
inRole(lavern, adult).
inRole(lavern, individual).
inRole(ted, adult).
inRole(ted, individual).
inRole(j, adult).
inRole(j, individual).
inRole(jordon, adult).
inRole(jordon, individual).


inRole(seattle_grace_hospital, covered_entity).
%inRole(the_office, entity).
inRole(hcp, healthCare_provider).
inRole(hoa, health_oversight_agency).
inRole(pha, public_health_authority).
inRole(hcao, healthCare_accreditation_organization).
inRole(hp, health_plan).
inRole(ghp, group_health_plan).

inRole(gahp, government_agency_health_plan).
inRole(hii, health_insurance_issuer).
inRole(hcch, healthCare_clearing_house).
	
inRole(patient, individual).
inRole(patient, adult).
inRole(thief, suspected_crime_perpetrator).
inRole(thief, individual).
inRole(thief, adult).
inRole(teen, emancipated_minor).
inRole(teen, individual).
inRole(kid, unemancipated_minor).
inRole(kid, individual).
inRole(mom, individual).
inRole(mom, adult).
inRole(dad, individual).
inRole(dad, adult).
inRole(dead, deceased_individual).
inRole(dead, individual).
inRole(dead, adult).
inRole(clergy_mem,clergy).

inRole(cop, law_enforcement_officer).
inRole(government_official, government_secretary).

%%relations
employee_of(dr_cox, sacred_heart_hospital).
employee_of(dr_kelso, sacred_heart_hospital).
employee_of(dr_turk, sacred_heart_hospital).
employee_of(dr_jd, sacred_heart_hospital).
employee_of(dr_elliot, sacred_heart_hospital).
employee_of(carla, sacred_heart_hospital).
employee_of(lavern, sacred_heart_hospital).
employee_of(ted, sacred_heart_hospital).
employee_of(j, sacred_heart_hospital).
employee_of(jordon, sacred_heart_hospital).

%Purposes of specifics.
purpose( surgery, treatment).

%business_assocate(X,Y):  Y is a business associate of X.
business_associate(the_office, sacred_heart_hospital).
business_associate(patient_safety_organization, sacred_heart_hospital).
business_associate(subcontractor, business_associate).
business_associate(health_information_exchange_organization, sacred_heart_hospital).
business_associate(E-prescribing_gateway, sacred_heart_hospital).
business_associate(regional_health_information_organization, sacred_heart_hospital).


in_relation(mom, seattle_grace_hospital).

personal_representative(dead, patient).
parent(kid, mom).
parent(kid, dad).
personal_representative(teen, dad).
personal_representative(teen, mom).
personal_representative(dad, mom).
personal_representative(mom, dad).
guardian(null, null).
loco_parentis(null, null).

relationship(mom, relative, teen).
involved(dad,payment,kid). 

test :-
  pbh(a(dr_cox,carla,carla,phi,create_deidentified_info,null,null,null)).

test :-
  pbh(a(dr_jd,dr_elliot,patient,phi,treatment,null,null,null)).
test :-
  writeln('First test;'),
  pbh(a(seattle_grace_hospital,null,patient,phi,health_record_sale,null,null,b(null,minimum_necessary_to_purpose,sacred_heart_hospital))).
test :-
  writeln('Second test;'),
  pbh(a(ted,lavern,patient,phi,determining_legal_options,null,null,b(sacred_heart_hospital,unlawful_covered_entity,lavern))).
test :-
  pbh(a(seattle_grace_hospital,sacred_heart_hospital,patient,phi,null,null,null,b(null,minimum_necessary_to_purpose,sacred_heart_hospital))).
test :-
  pbh(a(mom,carla,kid,phi,null,null,null,null)).
test :-
  pbh(a(ted,carla,kid,phi,payment,null,null,null)).
test :-
  pbh(a(teen,carla,thief,phi,treatment,null,null,null)).
test :-
  pbh(a(patient,carla,dead,phi,treatment,null,null,null)).
test :-
  pbh(a(dad,carla,teen,phi,treatment,null,null,null)).
test :-
  pbh(a(thief,carla,dr_kelso,phi,treatment,null,null,null)).
test :-
  pbh(a(seattle_grace_hospital,sacred_heart_hospital,dad,phi,create_protected_health_info,null,null,null)).
test :-
  pbh(a(sacred_heart_hospital,seattle_grace_hospital,dad,phi,create_protected_health_info,null,null,null)).
test :-
  pbh(a(sacred_heart_hospital,seattle_grace_hospital,dad,phi,create_protected_health_info,null,null,null)).

%Remove writeln when we don't want to see messages.  Set writeln to an AND of true or fail.  
%True if we want to assume unimplemented portions of code pass.
%Fail if we want to assume unimplemented portions of code always fail.
printY(X) :-
  writeln(X).
