package crypto;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.security.SecureRandom;
import java.security.spec.AlgorithmParameterSpec;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Set;
import sun.misc.BASE64Encoder;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;

import policy.HIPAAQuery;
import policy.HIPAAResult;
import security.abemodule.encryption.DocumentEncryptor;
import security.abemodule.utility.HipaaResultFormatter;
import server.process.ABERequest;
import settings.HIPAA_ABE_Settings;

public class EncryptionModule {	
	public String encryptedKeyFileName;// name of the des key of the file
	public String encryptedKeyPath;// path of des key
	//public String toBeEncryptedFileName;// path and name of unencrypted file
	public String encryptedFileName;// path and name of encrypted file (encrypted using des key)
	public String abeEncryptedKeyName; //name of abe encrypted key of des key
	public HIPAAQuery q;
	
	@SuppressWarnings("unchecked")
	public int encrypt(String message, ABERequest abeRequest) throws Exception{
		generateAESkey(message);
		
		Set<HIPAAResult> possibleResults = processQuery();
		int count = doABEEncryption(possibleResults, abeRequest);
		
		return count;
	}

	private int doABEEncryption(Set<HIPAAResult> possibleResults, ABERequest abeRequest)
			throws Exception {
		HipaaResultFormatter h = new HipaaResultFormatter(possibleResults);
		DocumentEncryptor documentencryptor = new DocumentEncryptor(HIPAA_ABE_Settings.locationofABETools,encryptedKeyFileName);	
		documentencryptor.gotokeydir();
		ArrayList<String> attributeList = h.stringFormat();
		int count = documentencryptor.encryptDocumentUsingAttributeList(attributeList, abeEncryptedKeyName, abeRequest);
		return count;
	}

	private Set<HIPAAResult> processQuery() {
		long startTime = new Date().getTime();
		HashMap<String, Set<HIPAAResult>> policy = null;
		try {
			InputStream in = this.getClass().getResourceAsStream(HIPAA_ABE_Settings.policyStoreFileName);
		    ObjectInputStream oi = new ObjectInputStream(in);
		    policy = (HashMap<String, Set<HIPAAResult>>) oi.readObject();
		} catch (Exception e) {
			e.printStackTrace();
		}
		//System.out.println("query hash = " + q.toString());
		Set<HIPAAResult> possibleResults = policy.get(q.toString());		
		long endTime = new Date().getTime();
		long total = endTime - startTime;
		System.out.println("Policy time:" + total + " ms");
		System.out.println(possibleResults);
		return possibleResults;
	}

	private void generateAESkey(String message) {
		SecretKey key = null;
		try {
			// generate AES key
			key = KeyGenerator.getInstance("AES").generateKey();
			
			//create random IV for CBC mode
			SecureRandom random = new SecureRandom();
			byte[] iv = new byte[16];
			random.nextBytes(iv);
			AlgorithmParameterSpec paramSpec = new IvParameterSpec(iv);
			
			//encrypt plaintext using AES
			Cipher ecipher = Cipher.getInstance("AES/CBC/PKCS5Padding");
			ecipher.init(Cipher.ENCRYPT_MODE, key, paramSpec);
			byte[] cipherText = ecipher.doFinal(message.getBytes());
			
			//store in cipherText and IV into a file
			System.out.println("Output path is " + encryptedKeyPath);
			File dir = new File(encryptedKeyPath);
			dir.mkdirs();
			
			ABEMessage msg = new ABEMessage();
			msg.IVbytes = ecipher.getIV();
			msg.messageBytes = cipherText;
			
			try {
				ObjectOutputStream oo = new ObjectOutputStream(new FileOutputStream(encryptedFileName));
				oo.writeObject(msg);
			} catch(Exception e) {
				System.err.println("Failed to write object to " + encryptedFileName);
				throw new Exception();
			}
					
			//store encrypted AES key in file
			FileOutputStream out2 = new FileOutputStream(encryptedKeyFileName);
			byte keyBytes[] = key.getEncoded();
			String base64Key = new String((new BASE64Encoder()).encode(keyBytes));
			String finalKey = "Key-" + base64Key;
			out2.write(finalKey.getBytes());
			out2.close();
			System.out.println(finalKey);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
