package security.abemodule.encryption;

import java.io.File;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import security.abemodule.utility.CommandExecutor;
import server.process.ABERequest;
import settings.HIPAA_ABE_Settings;


//Example Command for Document Encryption
//cpabe-enc pub_key security_report.pdf
//(sysadmin and (hire_date < 946702800 or security_team)) or
//(business_staff and 2 of (executive_level >= 5, audit_group, strategy_team))

public class DocumentEncryptor {
	private String keydirpath;
	private String documentName;

	public DocumentEncryptor(String keydirpath, String documentName){
		this.keydirpath = keydirpath;
		this.documentName = documentName;
	}

	public void gotokeydir(){
		CommandExecutor commandexecutor = new CommandExecutor();
		commandexecutor.setCommand("cd "+keydirpath);
		commandexecutor.executeCommand();
	}

	public void encryptDocument(String attributes, String encrypteddocname) throws Exception{
		CommandExecutor commandexecutor = new CommandExecutor();
		String fulldocumentPath = documentName; 
		String encDocPath = encrypteddocname; 
		attributes = "\""+attributes+"\"";
		String encryptcommand =  "./abe-enc -m CP -i "+ fulldocumentPath +" -p "+attributes +" -o "+encDocPath;
		String[] commands = new String[]{"cd "+keydirpath,encryptcommand};        
		commandexecutor.setCommand(commands);
		commandexecutor.executeCommand();
		
		File encOutFile = new File(encDocPath + ".cpabe");
		if(!encOutFile.exists()) {
			System.err.println("Output file " + encDocPath + " not created... bailing out");
			System.err.println("Tools path: " + HIPAA_ABE_Settings.locationofABETools);
			System.err.println("Did you copy d224.param to your fenc installation path?");
			System.err.println("Did you run \"./abe-setup -m CP\"?");
			throw new Exception();
		}
	}
	
	volatile int numRunningThreads = 0;
	volatile int numcompleted = 0; 
	
	class DocumentEncryptorThread extends Thread {
		String myAttributes;
		String myEncryptedDocName;
		long total;
		ABERequest abeRequest;
		
		public DocumentEncryptorThread(String attributes, String encrypteddocname, ABERequest req, long tot) {
			myAttributes = attributes;
			myEncryptedDocName =  encrypteddocname;
			abeRequest = req;
			total = tot;
		}
		
		public void run() {
			try {
				encryptDocument(myAttributes, myEncryptedDocName);
				
				int pctCompleted = (int)(100 * (((float)numcompleted++)/(float)total));
				abeRequest.setCompletion(pctCompleted);
				System.out.println("Completion: " + pctCompleted);
			} catch( Exception e ) {
				System.err.println("Encryption thread failed");
				e.printStackTrace();
			}
			numRunningThreads--;
		}
	}
	
	public int encryptDocumentUsingAttributeList (ArrayList<String> attributeList, String encrypteddocname, ABERequest abeRequest) throws Exception{
		int count = 0;
		int total = attributeList.size();
		abeRequest.setNumjobs(total);
		NumberFormat form = new DecimalFormat("000");
		
		for (String currentSet : attributeList){
			if(HIPAA_ABE_Settings.useThreading) {
				DocumentEncryptorThread dthread = new DocumentEncryptorThread(currentSet, encrypteddocname+form.format(count), abeRequest, total);
				
				while(numRunningThreads > HIPAA_ABE_Settings.maxThreads) {
					try {
						Thread.sleep(100);
					} catch(Exception e) {}
				}
					
				dthread.start();
				numRunningThreads++;
			} else {
				encryptDocument(currentSet, encrypteddocname+form.format(count));
			}
			count++;
		}

		if(HIPAA_ABE_Settings.useThreading) {
			while(numRunningThreads > 0) {
				try {
					Thread.sleep(200);
				} catch (InterruptedException e) {
					//e.printStackTrace();
				}
			}
		}
		
		return count;
	}
}
